#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# Copyright (c) 2017 Mozilla Corporation
# Contributors: Guillaume Destuynder <kang@mozilla.com>

import argparse
import glob
import json
import logging
import os
import sys
from authzero import AuthZero,AuthZeroRule

class DotDict(dict):
    """return a dict.item notation for dict()'s"""

    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __init__(self, dct):
        for key, value in dct.items():
            if hasattr(value, 'keys'):
                value = DotDict(value)
            self[key] = value

if __name__ == "__main__":
    # Logging
    formatstr="[%(asctime)s] %(levelname)s [%(name)s.%(funcName)s:%(lineno)d] %(message)s"
    logging.basicConfig(format=formatstr, datefmt="%H:%M:%S", stream=sys.stdout)
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)

    # Default credentials loading
    try:
        with open('credentials.json', 'r') as fd:
            credentials = DotDict(json.load(fd))
            require_creds = False
    except FileNotFoundError:
        credentials = DotDict({'client_id': '', 'client_secret': '', 'uri': 'auth-dev.mozilla.auth0.com'})
        require_creds = True

    # Arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-u', '--uri', default=credentials.uri, help='URI to Auth0 management API')
    parser.add_argument('-c', '--clientid', default=credentials.client_id, required=require_creds, help='Auth0 client id')
    parser.add_argument('-s', '--clientsecret', default=credentials.client_secret, required=require_creds, help='Auth0 client secret')
    parser.add_argument('-r', '--rules-dir', default='rules', help='Directory containing rules in Auth0 format')
    args = parser.parse_args()

    config = DotDict({'client_id': args.clientid, 'client_secret': args.clientsecret, 'uri': args.uri})
    authzero = AuthZero(config)
    authzero.get_access_token()
    logger.debug("Got access token for client_id:{}".format(args.clientid))

    # on any error, `authzero` will raise an exception and python will exit with non-zero code

    # Remote rules loader
    remote_rules = authzero.get_rules()
    logger.debug("Loaded {} remote rules from current Auth0 deployment".format(len(remote_rules)))

    # Local rules loader
    if not os.path.isdir(args.rules_dir):
        raise Exception('NotARulesDirectory' (args.rules_dir))

    # Process all local rules
    local_rules_files = glob.glob("{}/*.json".format(args.rules_dir))
    local_rules = []
    for rfile in local_rules_files:
        logger.debug("Reading local rule configuration {}".format(rfile))
        rule = AuthZeroRule()
        # Rule name comes from the filename with the auth0 format
        rule.name = rfile.split('/')[-1].split('.')[:-1][0]
        with open(rfile, 'r') as fd:
            rule_conf = DotDict(json.load(fd))
        rule.enabled = bool(rule_conf.enabled)
        rule.order = int(rule_conf.order)

        rcfile = rfile.rstrip('on') # Equivalent to s/blah.json/blah.js/
        logger.debug("Reading local rule code {}".format(rcfile))
        with open(rcfile, 'r') as fd:
            rule.script = fd.read()

        # Match with existing remote rule if we need to update.. this uses the rule name!
        rule_nr = [i for i,_ in enumerate(remote_rules) if _.get('name') == rule.name]
        if rule_nr:
            # If there's multi matches it means we have duplicate rule names and  we're screwed.
            # To fix that we'd need to change the auth0 local format to use rule ids (which we could eventually)
            if (len(rule_nr) > 1):
                raise Exception('RuleMatchByNameFailed', (rule.name, rule_nr))
            rule.id = remote_rules[rule_nr[0]].get('id')
            rule.is_new = False
        else:
            # No remote rule match, so it's a new rule
            logger.debug('Rule only exists locally, considered new and to be created: {}'.format(rule.name))
            rule.is_new = True

        if not rule.validate():
            logger.error('Rule failed validation: {}'.format(rule.name))
            sys.exit(127)
        else:
            local_rules.append(rule)
    logger.debug("Found {} local rules".format(len(local_rules)))

    # Find dead rules (i.e. to remove/rules that only exist remotely)
    remove_rules = []
    for rl in local_rules:
        ok = False
        for rr in remote_rules:
            if (rl.id == rr.get('id')) or (rl.is_new):
                ok = True
                continue
        if not ok:
            remove_rules.append(rr)
    logger.debug("Found {} rules that not longer exist locally and will be deleted remotely".format(len(remove_rules)))

    # Update or create (or delete) rules as needed
    ## Delete first in case we need to get some order numbers free'd
    for r in remove_rules:
        logger.debug("Removing rule {} ({}) from Auth0".format(r.name, r.id))
        authzero.delete_rule(r.id)

    ## Update & Create (I believe this may be atomic swaps for updates)
    for r in local_rules:
        if r.is_new:
            logger.debug("Creating new rule {} on Auth0".format(r.name))
            ret = authzero.create_rule(r)
            logger.debug("New rule created with id {}".format(ret.get('id')))
        else:
            logger.debug("Updating rule {} ({}) on Auth0".format(r.name, r.id))
            authzero.update_rule(r.id, r)

    sys.exit(0)



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# Copyright (c) 2017 Mozilla Corporation
# Contributors: Guillaume Destuynder <kang@mozilla.com>

import argparse
import difflib
import glob
import json
import logging
import logging.handlers
import os
import sys
from authzero import AuthZero,AuthZeroRule

class DotDict(dict):
    """return a dict.item notation for dict()'s"""

    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __init__(self, dct):
        for key, value in dct.items():
            if hasattr(value, 'keys'):
                value = DotDict(value)
            self[key] = value

def find_client_by_id(clients, client_id):
    """
    client_id: str - a client id
    clients: list - list of client objects
    Returns: client object or None
    """
    for c in clients:
        if (c.get('client_id') == client_id):
            return c
    return None

if __name__ == "__main__":
    # Default credentials loading
    try:
        with open('credentials.json', 'r') as fd:
            credentials = DotDict(json.load(fd))
            require_creds = False
    except FileNotFoundError:
        credentials = DotDict({'client_id': '', 'client_secret': '', 'uri': 'auth-dev.mozilla.auth0.com'})
        require_creds = True

    # Arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-u', '--uri', default=credentials.uri, help='URI to Auth0 management API')
    parser.add_argument('-c', '--clientid', default=credentials.client_id, required=require_creds, help='Auth0 client id')
    parser.add_argument('-s', '--clientsecret', default=credentials.client_secret, required=require_creds, help='Auth0 client secret')
    parser.add_argument('-d', '--debug', action="store_true", help='Enable debug mode')
    parser.add_argument('-v', '--verbose', action="store_true", help='Show log messages on stderr instead of sending to syslog')
    parser.add_argument('-r', '--clients-dir', default='clients', help='Directory containing clients in Auth0 format')
    parser.add_argument('--show-diff', action="store_true", help='Show a unified diff of the client being updated on stdout')
    parser.add_argument('-g', '--get-clients-only', action="store_true",
                        help='Get clients from the Auth0 deployment and write them to disk. This will make NO changes '
                        'to Auth0, but will OVERWRITE all local client files')
    args = parser.parse_args()

    # Logging
    logger = logging.getLogger(__name__)
    if not args.verbose:
        logger.addHandler(logging.handlers.SysLogHandler(address='/dev/log'))
    else:
        formatstr="[%(asctime)s] %(levelname)s [%(name)s.%(funcName)s:%(lineno)d] %(message)s"
        logging.basicConfig(format=formatstr, datefmt="%H:%M:%S", stream=sys.stderr)

    if args.debug:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    config = DotDict({'client_id': args.clientid, 'client_secret': args.clientsecret, 'uri': args.uri})
    authzero = AuthZero(config)
    authzero.get_access_token()
    logger.debug("Got access token for client_id:{}".format(args.clientid))

    # on any error, `authzero` will raise an exception and python will exit with non-zero code

    # Remote clients loader
    remote_clients = authzero.get_clients()
    logger.debug("Loaded {} remote clients from current Auth0 deployment".format(len(remote_clients)))

    if not os.path.isdir(args.clients_dir):
        raise Exception('NotAClientsDirectory' (args.clients_dir))


    # Write remote clients back to disk/local
    if args.get_clients_only:
        for client in remote_clients:
            with open("{}/{}.json".format(args.clients_dir, client.get('client_id')), 'w') as fd:
                fd.write(json.dumps(client, indent=4))
                logger.debug("Wrote local client configuration {}".format(client.get('client_id')))
        sys.exit(0)

    # Process all local clients
    local_clients_files = glob.glob("{}/*.json".format(args.clients_dir))
    local_clients = []
    for rfile in local_clients_files:
        logger.debug("Reading local clients configuration {}".format(rfile))
        with open(rfile, 'r') as fd:
            client = DotDict(json.load(fd))

        # Match with existing remote client to see if we need to update
        client_nr = [i for i,_ in enumerate(remote_clients) if _.get('client_id') == client.client_id]
        if client_nr:
            # Just in case we have dupe client_id's (this is not supposed to be possible)
            if (len(client_nr) > 1):
                raise Exception('ClientMatchByIdFailed', (client.name, client_nr))
        else:
            logger.debug('Client only exists locally, considered new and to be created: {}'.format(rule.name))

        local_clients.append(client)
    logger.debug("Found {} local clients".format(len(local_clients)))

    # Find dead clients (i.e. to remove clients that only exist remotely)
    remove_clients = []
    for rl in local_clients:
        if (find_client_by_id(local_clients, rl.client_id) is None):
            remove_clients.append(rr)
            continue
    logger.debug("Found {} clients that not longer exist locally and will be deleted remotely".format(len(remove_clients)))

    # Update or create (or delete) clients as needed
    for r in remove_clients:
        logger.debug("Removing remote client {} ({}) from Auth0".format(r.name, r.client_id))
        authzero.delete_client(r.client_id)

    ## Update & Create (I believe this may be atomic swaps for updates)
    for lclient in local_clients:
        if not lclient.client_id:
            logger.debug("Creating new remote client {} on Auth0".format(lclient.name))
            noop
            ret = authzero.create_client(r)
            logger.info("New remote client {} ({}) created on Auth0".format(ret.get('client_id'), ret.get('name')))
        else:
            rclient = find_client_by_id(remote_clients, lclient.client_id)
            if (rclient == lclient):
                logger.debug("Remote and local clients for id {} ({}) are identical, no need to "
                             "update".format(lclient.client_id, lclient.name))
                continue

            # Show pretty informational unified diff if requested, of the changes to be pushed to Auth0
            if args.show_diff:
                diff = difflib.unified_diff(json.dumps(rclient, indent=4).split('\n'),
                                            json.dumps(lclient, indent=4).split('\n'),
                                            fromfile="previous_{}.json".format(lclient.client_id),
                                            tofile="new_{}.json".format(lclient.client_id))
                print("Unified diff for client {} ({})".format(lclient.client_id, lclient.name))
                print('\n'.join(diff))

            logger.debug("Updating remote client {} ({}) on Auth0".format(lclient.name, lclient.client_id))
            authzero.update_client(lclient.client_id, lclient)
            logger.info("Client {} ({}) has been updated on Auth0".format(client.name, lclient.client_id))

    sys.exit(0)



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# Copyright (c) 2017 Mozilla Corporation
# Contributors: Guillaume Destuynder <kang@mozilla.com>

import argparse
import json
import logging
import sys
from authzero import AuthZero

class DotDict(dict):
    """return a dict.item notation for dict()'s"""

    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __init__(self, dct):
        for key, value in dct.items():
            if hasattr(value, 'keys'):
                value = DotDict(value)
            self[key] = value

if __name__ == "__main__":
    # Logging
    formatstr="[%(asctime)s] %(levelname)s [%(name)s.%(funcName)s:%(lineno)d] %(message)s"
    logging.basicConfig(format=formatstr, datefmt="%H:%M:%S", stream=sys.stdout)
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)

    # Default credentials loading
    try:
        with open('credentials.json', 'r') as fd:
            credentials = DotDict(json.load(fd))
            require_creds = False
    except FileNotFoundError:
        credentials = DotDict({'client_id': '', 'client_secret': '', 'uri': 'auth-dev.mozilla.auth0.com'})
        require_creds = True

    # Arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-u', '--uri', default=credentials.uri, help='URI to Auth0 management API')
    parser.add_argument('-c', '--clientid', default=credentials.client_id, required=require_creds, help='Auth0 client id')
    parser.add_argument('-s', '--clientsecret', default=credentials.client_secret, required=require_creds, help='Auth0 client secret')
    parser.add_argument('--default-client', default='VNGM4quJw3Nhx28j8XKVYmu5LcPMCgAH',
                        help='Default Auth0 client id, needed for login page for example')
    parser.add_argument('--login-page', required=True, help='Auth0 hosted login page (HTML)')
    args = parser.parse_args()

    config = DotDict({'client_id': args.clientid, 'client_secret': args.clientsecret, 'uri': args.uri})
    authzero = AuthZero(config)
    authzero.get_access_token()
    logger.debug("Got access token for client_id:{}".format(args.clientid))

    client_attributes = DotDict(dict())
    with open(args.login_page, 'r', encoding='utf-8') as fd:
        client_attributes.custom_login_page = fd.read()
    # on any error, `authzero` will raise an exception and python will exit with non-zero code
    ret = authzero.update_client(args.default_client, client_attributes)
    logger.debug("Default client updated {}".format(ret))
    sys.exit(0)



#!/usr/bin/env python
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
# Copyright (c) 2018 Mozilla Corporation
# Author: gdestuynder@mozilla.com

import os
from setuptools import setup


setup(
    name="Auth0-ci",
    version="1.0.3",
    author="Guillaume Destuynder",
    author_email="gdestuynder@mozilla.com",
    py_modules=["uploader_login_page", "uploader_rules", "uploader_clients"],
    description=("Super simple CI scripts for Auth0"),
    license="MPL",
    keywords="auth0 ci deploy",
    url="https://github.com/mozilla-iam/auth0-ci",
    install_requires=['authzerolib'],
    scripts=['uploader_clients.py', 'uploader_rules.py', 'uploader_login_page.py'],
    classifiers=["Development Status :: 5 - Production/Stable",
                 "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)"]
)
